﻿using System.Collections.Generic;
using NUnit.Framework;
using NUnit.Framework.SyntaxHelpers;

namespace CacheProject.Tests
{
    [TestFixture]
    public class DataLayerCacheTest 
    {
        [Test]
        public void Get_ShouldAddToCache()
        {
            // Arrange
            const string key = "TestData";

            // Act
            AddTestItem(key);

            // Assert
            bool exists = CacheLayer.Exists(key);

            Assert.That(exists, Is.True, "The cache does not contain the object");
        }

        [Test]
        public void GetAll_ReturnsResults()
        {
            // Arrange
            const string key = "TestData";

            // Act
            List<string> cachedData = CacheLayer.GetAll();

            // Assert
            Assert.That(cachedData.Count, Is.GreaterThan(0));
        }

        [Test]
        public void Exists_ShouldReturnFalse()
        {
            // Arrange
            const string key = "TestData";

            // Act
            bool exists = CacheLayer.Exists(key);

            // Assert
            Assert.That(exists, Is.False, "The cache is not supposed to contain the object");
        }

        [Test]
        public void Get_ShouldReturnItem()
        {
            // Arrange
            const string key = "TestData";

            // Act
            AddTestItem(key);
            List<string> list = CacheLayer.Get<List<string>>(key);

            // Assert
            Assert.That(list != null, "The cache does not contain the object");
            Assert.That(list.Count, Is.GreaterThan(0), "The cache does not contain the object");
        }

        [Test]
        public void Clear_ShouldRemoveItemFromCache()
        {
            // Arrange
            const string key = "TestData";

            // Act
            CacheLayer.Clear(key);

            // Then check if we have deleted it
            bool exists = CacheLayer.Exists(key);

            // Assert
            Assert.That(exists, Is.False, "The cache is not supposed to contain the object");
        }

        public List<string> CachingExample()
        {
            const string cacheKey = "CustomerNames";

            // Check the cache
            List<string> customerNames = CacheLayer.Get<List<string>>(cacheKey);

            if (customerNames == null)
            {
                // Go and retrieve the data from the DB
                customerNames = GetCustomerNames();

                // Then add it to the cache so we 
                // can retrieve it from there next time
                CacheLayer.Add(customerNames, cacheKey);
            }

            return customerNames;
        }

        public List<string> GetCustomerNames()
        {
            return new List<string> { "String1", "String2", "String3" };
        }

        public void AddTestItem(string key)
        {
            List<string> testData = new List<string> { "String1", "String2", "String3" };
            CacheLayer.Add(testData, key);
        }
            
    }
}

